@@ -1,19 +1,18 @@
 ################################################################################
 # File:             ddalpha-internal.r
-# Created by:       Pavlo Mozharovskyi
+# Created by:       Pavlo Mozharovskyi, Oleksii Pokotylo
 # First published:  28.02.2013
-# Last revised:     13.11.2015
+# Last revised:     20.02.2019
 # 
 # Contains the internal functions of the DDalpha-classifier.
 # 
 # For a description of the algorithm, see:
-#   Lange, T., Mosler, K. and Mozharovskyi, P. (2012). Fast nonparametric 
+#   Lange, T., Mosler, K. and Mozharovskyi, P. (2014). Fast nonparametric 
 #     classification based on data depth. Statistical Papers.
-#   Mozharovskyi, P., Mosler, K. and Lange, T. (2013). Classifying real-world 
+#   Mozharovskyi, P., Mosler, K. and Lange, T. (2015). Classifying real-world 
 #     data with the DDalpha-procedure. Mimeo.
 ################################################################################
 
-
 .ddalpha.create.structure <- function(formula, data, subset, ...){
   
   # if the user calls ddalpha(data, <other named parameters>, ...)
@@ -26,7 +25,7 @@
   # formula is present
   if(!missing(formula) && !is.null(formula)){
     
-    needed.frame <- sys.nframe() - 1
+    needed.frame <- 1 # PM(2018-06-22)
 
     cl <- match.call(call = sys.call(sys.parent(n = needed.frame)))
     mf <- match.call(expand.dots = FALSE, call = sys.call(sys.parent(n = needed.frame)))
@@ -148,6 +147,22 @@
   return (ddalpha)
 }
 
+.check.depth.exists <- function(depth) {
+  fname = paste0(".", depth, "_validate")
+  f <- try(match.fun(fname), silent = T)
+  if (!is.function(f))
+    warning(paste0("No validator function: ", fname))
+  fname = paste0(".", depth, "_learn")
+  
+  f <- (match.fun(fname))
+  if (!is.function(f))
+    stop(paste0("No function: ", fname))
+  fname = paste0(".", depth, "_depths")
+  f <- (match.fun(fname))
+  if (!is.function(f))
+    stop(paste0("No function: ", fname))
+}
+
 .ddalpha.learn.depth <- function(ddalpha){
   
   # try to find a custom depth
@@ -194,23 +209,24 @@
     } else
     if (ddalpha$methodDepth == "Mahalanobis"){
       ddalpha$patterns[[i]]$depths <- .Mahalanobis_depths(ddalpha, ddalpha$patterns[[i]]$points)
-    }
+    } else
     if (ddalpha$methodDepth == "spatial"){
       # Calculate depths for the class w.r.t all classes, saying to which of the classes the chunk belongs
       ddalpha$patterns[[i]]$depths <- .spatial_depths(ddalpha, ddalpha$patterns[[i]]$points)
-    }
+    } else
     if (ddalpha$methodDepth == "spatialLocal"){
       # Calculate depths for the class w.r.t all classes, saying to which of the classes the chunk belongs
       ddalpha$patterns[[i]]$depths <- .spatialLocal_depths(ddalpha, ddalpha$patterns[[i]]$points)
-    }
+    } else
     if (ddalpha$methodDepth == "simplicial"){
       # Calculate depths for the class w.r.t all classes, saying to which of the classes the chunk belongs
       ddalpha$patterns[[i]]$depths <- .simplicial_depths(ddalpha, ddalpha$patterns[[i]]$points)
-    }
+    } else
     if (ddalpha$methodDepth == "simplicialVolume"){
       # Calculate depths for the class w.r.t all classes, saying to which of the classes the chunk belongs
       ddalpha$patterns[[i]]$depths <- .simplicialVolume_depths(ddalpha, ddalpha$patterns[[i]]$points)
-    }
+    } else
+      stop("Unknown depth ", ddalpha$methodDepth)
   }
 
   return (ddalpha)
@@ -344,6 +360,7 @@
 }
 
 .polynomial_classify <- function(ddalpha, classifier, depths){
+
   x = ifelse(classifier$axis == 0, classifier$index1, classifier$index2)
   y = ifelse(classifier$axis == 0, classifier$index2, classifier$index1)
   
@@ -353,8 +370,6 @@
     for(j in 1:classifier$degree){
       result[obj] <- result[obj] + classifier$polynomial[j]*val^j
     }
-    if(classifier$axis != 0)
-      result[obj] <- (- result[obj])
   }
   return(result)
 }
@@ -502,7 +517,8 @@
 
   .treatment = treatments[[settings$method]]
   if(is.null(.treatment)) stop("Unknown outsiders treatment method ", settings$method)
-  return(.treatment(objects, ddalpha, settings))
+  classified = .treatment(objects, ddalpha, settings)
+  return(classified)
 }
 
 ################################################################################
@@ -817,6 +833,13 @@
     
     points <- as.vector(t(pattern))
     x <- as.vector(t(objects))
+    if (ddalpha$d_useCov == 0){
+      covEst <- diag(ncol(pattern))
+    } else if (ddalpha$d_useCov == 1){
+      covEst <- cov(pattern)
+    } else if (ddalpha$d_useCov == 2){
+      covEst <- covMcd(pattern, ddalpha$d_parMcd)$cov
+    }
     ds <- .C("OjaDepth", 
              as.double(points), 
              as.double(x), 
@@ -826,6 +849,8 @@
              as.integer(ddalpha$seed),
              as.integer(ddalpha$d_exact),
              as.integer(.longtoint(ddalpha$d_k)),
+             as.integer(ddalpha$d_useCov),
+             as.double(as.vector(t(covEst))),
              depths=double(nrow(objects)))$depths
     depths <- cbind(depths, ds, deparse.level = 0)
   }  
@@ -894,14 +919,14 @@
     pattern <- ddalpha$patterns[[i]]$points
     mean <- ddalpha$patterns[[i]]$center
     cov <- ddalpha$patterns[[i]]$cov
-    suppressWarnings(
-    if(!is.na(cov)){
+    #suppressWarnings(
+    if(sum(is.na(cov)) == 0){
       cov.eig <- eigen(cov)
       B <- cov.eig$vectors %*% diag(sqrt(cov.eig$values))
       lambda <- solve(B)
     } else{
       lambda = diag(ncol(pattern))
-    })
+    }#)
     ds <- rep(-1, nrow(objects))
     for (i in 1:nrow(objects)){
       tmp1 <- t(lambda %*% (objects[i,] - t(pattern)))
